import { EmitterShapeBase } from "@tsparticles/plugin-emitters";
import { errorPrefix } from "@tsparticles/engine";
import { generateRandomPointOnPathPerimeter, generateRandomPointWithinPath } from "./utils.js";
export class EmittersPathShape extends EmitterShapeBase {
    constructor(position, size, fill, options) {
        super(position, size, fill, options);
        const ctx = document.createElement("canvas").getContext("2d");
        if (!ctx) {
            throw new Error(`${errorPrefix} No 2d context available`);
        }
        this.checkContext = ctx;
        this.points = options.points;
        const pathData = this.points, path = new Path2D(), offset = {
            x: position.x - size.width / 2,
            y: position.y - size.height / 2,
        };
        for (const [index, point] of pathData.entries()) {
            const coords = {
                x: offset.x + (point.x * size.width) / 100,
                y: offset.y + (point.y * size.height) / 100,
            };
            if (index === 0) {
                path.moveTo(coords.x, coords.y);
            }
            else {
                path.lineTo(coords.x, coords.y);
            }
        }
        if (pathData[0]) {
            const coords = {
                x: offset.x + (pathData[0].x * size.width) / 100,
                y: offset.y + (pathData[0].y * size.height) / 100,
            };
            path.lineTo(coords.x, coords.y);
        }
        this.path = path;
    }
    async init() {
    }
    async randomPosition() {
        const ctx = this.checkContext, position = this.position, size = this.size, fill = this.fill, path = this.path, res = fill
            ? generateRandomPointWithinPath(ctx, path, position, size)
            : generateRandomPointOnPathPerimeter(ctx, path, position, size);
        return res ? { position: res } : null;
    }
    resize(position, size) {
        super.resize(position, size);
        const pathData = this.points, path = new Path2D(), offset = {
            x: position.x - size.width / 2,
            y: position.y - size.height / 2,
        };
        for (const [index, point] of pathData.entries()) {
            const coords = {
                x: offset.x + (point.x * size.width) / 100,
                y: offset.y + (point.y * size.height) / 100,
            };
            if (index === 0) {
                path.moveTo(coords.x, coords.y);
            }
            else {
                path.lineTo(coords.x, coords.y);
            }
        }
        if (pathData[0]) {
            const coords = {
                x: offset.x + (pathData[0].x * size.width) / 100,
                y: offset.y + (pathData[0].y * size.height) / 100,
            };
            path.lineTo(coords.x, coords.y);
        }
        this.path = path;
    }
}
